/* this file spawns a process to write to a pipe and spwans l3enc to read */
/* from this pipe */
/* several encoders are listed..... */

#include "mp3serv.h"
#include <sys/soundcard.h>

#define BUF_LENGTH 256
#define BUF_SIZE BUF_LENGTH*2
#define BUFLEN 1024*32

void read_soundcard(config conf,FILE *mnt){
int format,stereo,speed,len,blksize,i,j,k,mode,clip;
int audio_fd,audio_play_rate,vol;
short audiobuf[4096];
FILE *pipe;

/* this soundcard setup is optomised for my setup */
/* 16 bit signed, mono, */
printf("Initialising soundcard\n");
if((audio_fd=open("/dev/dsp",O_RDONLY))==-1)
  fatal("Failed to open sound device");
ioctl(audio_fd, SNDCTL_DSP_GETFMTS, &format);  
if(!(format&AFMT_S16_LE))
  printf("16bit mode not supported\n");
format = AFMT_S16_LE;
if(ioctl(audio_fd,SNDCTL_DSP_SETFMT,&format)==-1)
  fatal("Failed to set data format\n");
stereo=0;
if(ioctl(audio_fd,SNDCTL_DSP_STEREO,&stereo)==-1)
  fatal("Somethign went wrong with the stereo setting\n");
audio_play_rate=22050;
if(ioctl(audio_fd,SNDCTL_DSP_SPEED,&audio_play_rate)==-1)
  fatal("Speed Setting failed\n");

if((pipe=fopen(conf.snd_pipe,"wb"))==NULL)
  fatal("opening raw audio pipe for writing sound\n");

/* zero the first second of music to clean out the stream */
printf("Blanking first seconds of music\n");
j=1+(SAMPLE_RATE/4096);
printf("%d\n",j);
 for(i=0;i<4096;i++)
   audiobuf[i]=0;
 for(i=0;i<j;i++)
   fwrite(audiobuf,2,4096,pipe);
 

while(1){
  read(audio_fd,audiobuf,2*4096);
  fwrite(audiobuf,2,4096,pipe);
  vol=0;
  clip=0;
  for(i=0;i<4096;i++){
    if(abs(audiobuf[i])>vol)
      vol=abs(audiobuf[i]);
  }
  /* this is bigger so that it shows through the encoders output */
  fprintf(stderr,"\r                              \r%d ",vol);
  if(vol>32000)
    fprintf(stderr,"******clip*********");
}
close(audio_fd);
fprintf(stderr,"\n");
}

void play_mp3_list(config conf,FILE *mnt){
short stereo[4096],mono[2048];
unsigned long data_written; 
time_t start,curnt;
double dtime;
char sr[32];
short vol,clip; 
int i,j,fd[2],len;
FILE *spipe;
if((spipe=fopen(conf.snd_pipe,"wb"))==NULL)
  fatal("opening raw audio pipe for writing sound\n");


/* first we need a pipe for mpg123 to talk to us */
 pipe(fd);
 sprintf(sr,"%d",SAMPLE_RATE);
 switch(fork()){
 case -1:
   fatal("forking mpg123");
   break;
 case 0:
   /* run decoder as child */
   /* close stdout and link it to the pipe */
   close(1);
   dup(fd[1]);
   close(fd[1]);
   /* close the read end of the pipe */
   close(fd[0]);
   /* now execute the relevant decoder - random or fixed */  
    if(conf.server==2)
      execlp("mpg123","mpg123","-s","-@","mp3list.dat",NULL);
    else 
      execlp("mpg123","mpg123","-s","-Z","-@","mp3list.dat",NULL);
    fatal("can't exec mpg123");
    break;
 default:
   /* close pipe write end */
   close(fd[1]);
   /* zero out stream start */
   j=1+(SAMPLE_RATE/4096);
   for(i=0;i<4096;i++)
     stereo[i]=0;
   for(i=0;i<j;i++)
   fwrite(stereo,2,4096,spipe);
   /* need to know how much we've written to throttle stream */
   data_written=0;
   time(&start);
   /* now read the data forever */
   while(1){
     len=read(fd[0],stereo,4096);
     len = len >>2;
     vol=0;
     clip=0;
     /* slow and monoise the input */
     for(i=0;i<len;i++){
       mono[i]=(short)(((int)(stereo[i*4])+(int)(stereo[i*4]+1)+(int)(stereo[i*4+2])+(int)(stereo[i*4+3]))/4);
       if(abs(mono[i])>vol)
	 vol=abs(mono[i]);
     }
     fwrite (mono,2,len>>1,spipe);
     data_written+=len>>1;
     /* this is bigger so that it shows through the encoders output */
     fprintf(stderr,"\r                              \r%d %d",vol,len);
     if(vol>32000)
     fprintf(stderr,"******clip*********");
     /* now see if we're ahead and sleep */
     time(&curnt);
     dtime=difftime(curnt,start);
     if(data_written>22050*dtime){
       sleep(1);
     }
   }
   
 }
	 
	 

}



/* OK.... this is the routine to launch the old l3enc encoder */
/* It ONLY works with the registerd version */
/* The shareware version doesn't work because it needs a file with a */
/* header, and performs an fseek() on this - in theory you could hack */
/* the kernels so that pipes don't return an error on an fseek() attempt */
/* but...... I'm not going to condone breaking licensing agreements */
/* The quality produuced by the fraunhofer code is much better than */
/* that standard ISO sources - if you're at all serious about this radio */
/* business then you'll get some of their code which is the best that I've*/
/* found */
void exec_encoder(config conf,FILE *mnt){
char sr[32],br[32];
  /*isolate the IO so it doesn't clobber the level meter*/
close(0);
close(1);
close(2);
open("/dev/null",O_RDONLY);
open("/dev/null",O_WRONLY);
open("/dev/null",O_WRONLY);
sprintf(sr,"%d",SAMPLE_RATE);
sprintf(br,"%d",BITRATE);
execlp("l3enc","l3enc",conf.snd_pipe,conf.live_file,"-br",br,"-sr",sr,"-tfc","1","-dm",NULL);
fatal("failed to launch l3enc");
}

/* ok *in theory* this will work with fraunhofer's mp3enc */
/* I can;t really try it.... */
void exec_newencoder(config conf,FILE *mnt){
char iff[128],br[32];
sprintf(iff,"nc=1 bps=16 sr=%d ",SAMPLE_RATE);
sprintf(br,"%d",BITRATE);
execlp("mpenc","mpenc","-if",conf.snd_pipe,"-of",conf.live_file,"-iff",iff,"-br",br,NULL);
fatal("failed to launch mp3enc");
}


/* this is my own hacked up version using the dist10 sources */
/* this is *NOT* available right now for legal reasons */
/* this one is reading from the pipe even though it doesn't need to */
void exec_myencoder(config conf,FILE *mnt){
char sr[32],br[32];
sprintf(sr,"%d",SAMPLE_RATE);
sprintf(br,"%d",BITRATE);
execlp("mpenc","mpenc",sr,br,"1",conf.snd_pipe,conf.live_file,NULL);
fatal("failed to launch encoder");
}

/* If you want to do a minimal conversion of the dist10 sources then */
/* you need to remove the checks that exit when an fseek() fails it  */
/* doesn't force the programme to exit  */
void exec_dist10encoder(config conf,FILE *mnt){
printf("Launching Encoder\n");
/* gurr - there are too many rules to put into this layer without hard coding
   values - I'm sure you can figure it out for yourself.,....*/

}

 
/* forget this for thie moment - use the outside server */
void start_recording(config conf){
FILE *mnt,*snd;
if((mnt=fopen(conf.mnt_pipe,"w"))==NULL)
  fatal("opening mnt_pipe while spawning encoder");
switch (fork())
      {
      case 0:
	exec_encoder(conf,mnt);    
      case -1:
	fatal("fork() - with encoder");
      }
if(conf.server==1)
  read_soundcard(conf,mnt);
else
  play_mp3_list(conf,mnt);
}
